########################################################################
# ILP.pm - An object module for communication with Interactive ListProc
#
# WARNING: This software is experimental and still in development.
# Code may change at will!
#
# The latest version of this software can be found in:
#     http://pantheon.yale.edu/~cer28/projects/ilp/
#
# Copyright (C) 1999 Chad Redman, Yale University
#
#
# Comments, bug fixes, and feature requests to: chad.redman@yale.edu
#
########################################################################

package ILP;

# $Id: ILP.pm,v 0.6 1999/05/19 14:34:52 cer28 Exp $

$VERSION = do { my @r=(q$Revision: 0.6 $ =~ /\d+/g); sprintf "%d."."%02d"x$#r,@r };

$MAX_CMD_LENGTH = 1023;

use IO::Socket;

# A package variable that occasionally will get set. Most useful for
# displaying diagnostics when 'new' fails
my $ERRMSG = '';


### new( ILP, $host, $user, $pass )
#   -------------------------------
#   Create a new ILP object. Calling this will open a connection to $host
#   ($host can be 'host:port', 'host' (defaults to 'host:372'), or '' (defaults
#   to 'localhost:372'). Attempt to log in with $user and $pass (or log in as
#   a casual user if undefined)

sub new
{
	my($pkg, $host, $user, $pass) = @_;

	### check host for hostname:port syntax
	($host) or ($host = 'localhost');
	($host !~ /:\d+$/) and ($host = $host . ':372');

	my $self = bless {
		'host'          	=> $host,
		'user'          	=> $user,
		'pass'          	=> $pass,
		'sock'          	=> undef,
		'connect_cookie'	=> undef,
		'lp_version'    	=> undef,
		'timeout'       	=> undef,
		'login_type'		=> undef
	}, $pkg;

	$self->connect() or return undef;
	$self->login() or return undef;
	$self
}


### General accessors
sub host       { my $self = shift; $self->{'host'}          }
sub user       { my $self = shift; $self->{'user'}          }
sub pass       { my $self = shift; $self->{'pass'}          }
sub lp_version { my $self = shift; $self->{'lp_version'}    }
sub timeout    { my $self = shift; $self->{'timeout'}       }
sub login_type { my $self = shift; $self->{'login_type'}    }


### connect($ilp) returns boolean
#   At this point in development, this is not meant to be a public method
#   But conceivably, it could be called to reopen an ILP object closed by
#   $ilp->close

sub connect {
	my $self = shift;

	# my $sock = IO::Socket::INET->new( $self->{'host'} ); # does not work for IO::Socket version < 1.1603

	my ($host, $port) = split(':', $self->{'host'});
	$port = $port || 372;
	my $sock = IO::Socket::INET->new( PeerAddr=>$self->{'host'}, PeerPort=>$port, Proto=>'tcp' );

	if (!$sock) {
		$ILP::ERRMSG = 'Could not make connection to host ' . $self->{'host'} . '.';
		return undef;
	}

	$sock->autoflush;   # Needed for IO::Socket version < 1.1603
	$self->{'sock'} = $sock;
	chomp( $self->{'connect_cookie'} = $sock->getline );

	# Parse out the ListProc version from the connect cookie, of the
	# form (for example) '105 600 8.2.07 9 2'

	if ( $self->{'connect_cookie'} =~ /^\d+ (\d+) (\S+) \d+ \d+$/) {
		$self->{'timeout'} = $1;
		$self->{'lp_version'} = $2
	}

	my ($code, $response) = $self->response();	# skip the welcome message. Return value is the message
	($code == 100) or $ILP::ERRMSG = "$code:$response";
	return ($code==100)
};


#### login($ilp) returns string {manager|owner|subscriber|casual|???}
#    Sets: self->login_type
#          ILP::ERRMSG, on error (the code + response text upon error)
sub login {
	my $self = shift;
	my ($code, $response) = $self->request( $self->{user} );
	if ($code == 103) {
		($code, $response) = $self->request( $self->{pass} );
		$ERRMSG = "";
	} else {
		($code, $response) = $self->request();
		$ERRMSG = "$code:$response";
	}

	### Types of login messages:
	#You have logged in with manager privileges; you may not issue 'execute'
	#You have logged in with owner privileges; valid requests are:
	#You have logged in with subscriber privileges; you may only issue the following
	#You have logged in as a casual user; you may only issue the following requests:

	if ($response =~ /^You have logged in with (\w+) privileges;/) { $self->{login_type} = $1; }
	elsif ($response =~ /^You have logged in as a casual user;/) { $self->{login_type} = 'casual'; }
	else { $self->{login_type} = "??? ($response)"; }
	$self->{login_type}
}


### request($ilp, $cmd) returns ($code, $response)
#   Send the command in $cmd to the server as a single line. Capture the
#   code + response string, parse it, and return both values. This includes
#   the server prompt (usually 'request>'). To strip this out, call
#   do_command instead

sub request {
	my ($self, $cmd) = @_;

	# The ILP protocol allows a &\n to indicate continuing input. But
	# that would break the normal response expected here.
	# putting a space on the end of the line effectively escapes the line
	# continuation property of $\n
	( $cmd =~ /&$/s ) and ( $cmd .= ' ' );

	$ILP::DEBUG and print "command: $cmd\n";

	# Can't go over the command length limit. If attempted set ERRMSG
	# and return failure

	if ( length($cmd) > $MAX_CMD_LENGTH ) {
		$ILP::ERRMSG = "Length of command > $MAX_CMD_LENGTH";
		return (-100, $ILP::ERRMSG)
	}

	my $sock = $self->{'sock'};

	print $sock $cmd, "\n";

	my ($code, $res) = $self->response;

	($code, $res)
}


### response($ilp) returns ($code, $response)
#   After sending a command, retrieve all code that comes back. The ILP
#   protocol specifies that the first line of the response will give the
#   return code and the number of bytes in the response. The code is saved
#   for returning to the caller, and a loop reads in bytes until the length
#   is satisfied

sub response {
	my $self = shift;
	my $sock = $self->{'sock'};

	# The first line of the response gives the result code plus the
	# number of bytes in the response (including the 'request>' prompt)

	my ($code, $bytes_to_read);

	# For Casual login, the response is first (100,0), then (100, 885)
	# Thus, loop until a non-zero bytes_to_read is found
	do { ($code, $bytes_to_read) = split(' ', $sock->getline) } while $bytes_to_read == 0;

	# Loop through a socket read, until the right number of bytes is
	# obtained. It needs to be a loop, because a single read doesn't
	# seem to get it all at once (depends on network speed?)

	my ($msg, $buf, $bytes_read);
	while ( $bytes_to_read ) {
		$bytes_read = read($sock, $buf, $bytes_to_read);
		if ($buf eq '') {
			### closed connection
			$self->close;
			last;
		} else {
			$msg .= $buf;
			$bytes_to_read -= $bytes_read;
		}

	}

	($code, $buf)
}


### do_command($ilp, $cmd) returns ($code, $response)
#   A friendlier version of the request function. Issue the command in $cmd
#   to the server as a single line. Strip out unimportant text (the 'Your
#   command ... produced the following output' and the prompt 'request>')
#   and return everything else in the response.

sub do_command {
	my ($self, $command) = @_;
	
	if ( !defined( $self->{'sock'} )) {
		$ILP::ERRMSG = 'Not Connected.';
		return ()
	}

	# In do_command, only one command per function call is allowed.
	# This strips internal '&' continuation characters. A '&' on the
	# last line (possible security hole!) is taken care of in request()
	$command =~ s/&\n//gs;
	$command =~ s/\n/ /gs;

	my ($code, $resp) = $self->request( $command );
	$resp =~ s/^.*produced the following output:\n\n//s;
	$resp =~ s/\nrequest> $//s;
	$ILP::DEBUG and print "Response: code=$code | response='$resp'\n";
	($code, $resp);
}


### close($ilp) returns nothing
#   Issue a 'quit' to the server, and then drop the socket connection

sub close {
	my $self = shift;
	$self->request('quit'); # code should be 900, but close in any case
	$self->{'sock'}->close();
	$self->{'sock'}=undef;
}


#************************** User Information **************************#

### get_list_subs($ilp) returns array of lists to which the user is
###                          subscribed

sub get_list_subs {
	my $self = shift;
	my ($code, $resp) = $self->do_command('which');

	if ($code ne '100') {
		$ILP::ERRMSG = "$code:$resp";
		return ()
	}

	my @subs;
	if ($resp =~ /if none appear, you are not subscribed to any:\n(.*)/s) {
		@subs = split("\n", $1);
	}

	# Don't return the ($code, $response) style here, because ignoring
	# the $code return value won't work when the rest of the response is
	# an array. Instead, just return the array of lists. However, the
	# caller can't detect success based on returned results, because an
	# empty list may be valid. Instead, reset ERRMSG to null

	$ILP::ERRMSG = undef;
	@subs
}


### get_list_avail_names($ilp)
### get_list_avail_names($ilp, 'local')
###       returns array of available lists, with the descriptions stripped off
sub get_list_avail_names($ilp) {
	my ($self, $is_local) = @_;      # $is_local for 'LISTS LOCAL'
	my @lists = $self->get_list_avail( $is_local );

	($ILP::ERRMSG) and return undef;

	for ( 0 .. $#lists ) {
		$lists[$_] =~ s/\s.*//;
	}

	@lists
}
			
	

### get_list_avail($ilp) or get_list_avail($ilp, 'local')
###       returns array of available lists

sub get_list_avail {
	my ($self, $is_local) = @_;      # $is_local for 'LISTS LOCAL'

	# syntax checking
	if (($is_local) and (lc($is_local) ne 'local')) {
		$ILP::ERRMSG = "Unrecognized directive: LISTS " . uc($is_local);
		return undef
	}

	my ($code,$resp) = $self->do_command("lists $is_local") or return undef;

	if ($code ne '100') {
		$ILP::ERRMSG = "$code:$resp";
		return ()
	}
	
	my @avail;	

	if ($resp =~ /Here is the current active list of the (\d+) mailing lists served by this server:\n\n/) {
		@avail = grep( s/\s+/\t/, split("\n", $') );
		#@avail = grep( s/\s.*//, split("\n", $') );
		if (scalar(@avail) != $1) {
			warn "Warning {ILP::get_list_avail}: ", scalar(@avail),
			  " available lists found, but count given was $1\n";
		}
	}

	# Caller can't detect success based on returned results, because an
	# empty list may be valid. Instead, reset ERRMSG to null
	$ILP::ERRMSG = undef;
	@avail
}


### set_password($ilp, $list, $oldpass, $newpass) returns ($code, response)
#   Changes the user's password for the given list. A code of 100 indicates
#   success

sub set_password {
	my ($self, $list, $oldpass, $newpass) = @_;

	if ($newpass eq '-') {
		$ILP::ERRMSG = "Minus (-) sign not allowed as a password";
		return (-100, $ILP::ERRMSG)
	}

	if ($list =~ /\s/ or $oldpass =~ /\s/ or $newpass =~ /\s/) {
		# This is the same error as in a real ILP session
		$ILP::ERRMSG = "Too many SET parameters: SET $list PASSWORD $oldpass $newpass";
		return (-100, $ILP::ERRMSG)
	}

	my ($code, $resp) = $self->do_command("SET $list PASSWORD $oldpass $newpass");
	($code, $resp);
}


### set_address($ilp, $list, $pass, $newaddress) returns ($code, response)
#   Changes the user's address for the given list. A code of 100 indicates
#   success. I still need to investigate, but it seems that if you change
#   your address for one list, your login email address gets aliased to the
#   new one, and you can no longer change the address on other lists. Oops :(

sub set_address {
	my ($self, $list, $pass, $newaddress) = @_;

	if ($list =~ /\s/ or $pass =~ /\s/ or $newaddress =~ /\s/) {
		$ILP::ERRMSG = "Too many SET parameters: SET $list ADDRESS $pass $newaddress";
		return (-100, $ILP::ERRMSG)
	}

	my ($code, $resp) = $self->do_command("SET $list ADDRESS $pass $newaddress");
	($code, $resp);
}


### set_mail($ilp, $list, $mail_setting) returns ($code, response)
#   Changes the user's mail preference for a list. Choices are:
#   ACK|NOACK|DIGEST|POSTPONE

sub set_mail {
	my ($self, $list, $mail_setting) = @_;

	if ($list =~ /\s/ or $mail_setting =~ /\s/) {
		$ILP::ERRMSG = "Too many SET parameters: SET $list MAIL $mail_setting";
		return (-100, $ILP::ERRMSG)
	}

	my ($code, $resp) = $self->do_command("SET $list MAIL $mail_setting");
	($code, $resp);
}


### set_conceal($ilp, $list, $conceal) returns ($code, response)
#   Changes the user's conceal flag for a list. Choices are: YES|NO

sub set_conceal {
	my ($self, $list, $conceal) = @_;

	my $cmd = "SET $list CONCEAL $conceal";

	if ($list =~ /\s/ or $conceal =~ /\s/) {
		$ILP::ERRMSG = "Too many SET parameters: $cmd";
		return (-100, $ILP::ERRMSG)
	}

	my ($code, $resp) = $self->do_command( $cmd );
	($code, $resp);
}


### set_user_options($ilp, $list, $option, @arguments, [$option, $arguments] ...)
###           returns: array of responses from each change
#   This is now deprecated in favor of the set_mail, set_conceal, etc. functions.
#   However, I'll leave it in in case someone finds it useful. This is a way to
#   specify multiple subscriber changes with a single call. E.g.,
#   @resps = $ilp->set_user_options('password'=>'oldpass'=>'newpass',
#                             'mail'=>'ack', 'conceal'=>'yes')

sub set_user_options {
	my $self = shift;
	my $list = shift;
	my @responses;
	my ($code, $resp);

	while (@_) {
		my $option = shift;
		if ( $option =~ /^(mail|conceal)$/i ) {
			my $target = shift  or return @responses;
			($code, $resp) = $self->do_command("SET $list $option $target");
			push @responses, "$code:$resp";
		}
		elsif ( $option =~ /^(address|password)$/i ) {
			my $pass = shift  or return @responses;
			my $target = shift or return @responses;
			($code, $resp) = $self->do_command("SET $list $option $pass $target");
			push @responses, "$code:$resp";
		}
		else {
			warn "Unknown option: $option\n";
		}
	}
	@responses
}


### get_user_options($ilp, $list) returns hash of option=>value for the
###           specified list
### get_user_options($ilp, $list, $option) returns the value for the
###           specified option                       
#   Example 1: my %opts = $ilp->get_user_options('ANNOUNCE-L')
#   Example 2: my $pass = $ilp->get_user_options('ANNOUNCE-L', 'password')

sub get_user_options {
	my ($self, $list, $option) = @_;

	if ( ! $list ) {
		$ILP::ERRMSG = 'No list name given to get_user_options';
		return $option ? undef : ()
	}

	if ($list =~ /\s/) {
		$ILP::ERRMSG = "Too many parameters for SET list: $list";
		return $option ? undef : ()
	}

	my ($code, $resp) = $self->do_command("SET $list");

	if ($code ne '100') {
		$ILP::ERRMSG = "$code:$resp";
		return $option ? undef : ()
	}

	if ($resp =~ /Current settings are:\n+(.*)/s) {
		my @attrib;
		foreach ( split("\n", $1) ) {

			# Split the OPTION: VALUE into a hash pair
			(/ = /) and push @attrib, (lc($`), $');

			# If option specified and matches this key, return now
			( lc($`) eq lc($option) ) and return $';
		}

		# if an option was specified but not encountered
		if ($option) {
			$ILP::ERRMSG = "User option $option not defined";
			return undef
		}

		# otherwise return all pairs as a hash-compatible list
		return @attrib;
	}

	# Unexpected response
	$ILP::ERRMSG = "$code:$resp";
	return ();
}


#--------- List Information ----------#

### get_list_options($ilp, $list) returns hash of option=>value pairs
### get_list_options($ilp, $list, $option) returns value of specified option
# parse the list settings into an array of pairs. A sample array of settings may have the
# following keys: ( * = special parsing performed to obtain value )
#    *address
#    *description
#    *created
#    *info
#     open
#     send
#     visible
#     unpublished
#     no-archive
#     stats
#     review
#     archives
#     unmoderated
#     digest
#     message-limit
#     forward-rejects
#    *reply-to-list-always (boolean)
#     auto-delete-subscribers
#     keep-resent-lines
#     delivery-errors
#     owners

sub get_list_options {
	my ($self, $list, $option) = @_;

	if ( ! $list ) {
		$ILP::ERRMSG = 'No list name given to get_user_options';
		return $option ? undef : ()
	}


	if ($list =~ /\s/) {
		$ILP::ERRMSG = "Too many parameters for SET list: $list";
		return $option ? undef : ();     # return array or scalar as appropriate
	}


	my ($code, $resp) = $self->do_command("REV $list SHORT");

	if ($code ne '100') {
		$ILP::ERRMSG = "$code:$resp";
		return ()
	}

	my @attrib;
	my $mode='header';    # header | options | options_found | info
	my $info;

	foreach (split("\n", $resp)) {

		if ($mode eq 'info') {
			# ignore initial blank lines
			next if ( ! $info and $_ =~ /^\s*$/ );

			# Ignore this one too (only in v 8.?)
			next if ( ! $info and $_ eq '--- List Information' );
			
			$info .= $_ . "\n";
			next;
		}

		if ( /--- The current list settings are as follows/ ) {
			$mode = 'options';
			next
		}
			
		if ($mode eq 'header') {
			next if (/^\s*$/);    # blank line within header

			if ( /^\*\*\*\s+([^:]+): / ) {
				if ($1 eq 'Date created') {
					push (@attrib, 'created'=>$');
				}
				else {                # an 'address: description' line
					push (@attrib, 'address'=>$1, 'description'=>$')
				}
			} else { ; }                  # unrecognizable
		}

		if ($mode eq 'options' or $mode eq 'options_found') {
			
			if (/^([^:]+):\s*/) {
				push (@attrib, lc($1) => $');
				$mode = 'options_found';
			}
			elsif (/^([\w][- \w]*)$/) {   # a boolean setting
				push (@attrib, lc($1) => 1);
				$mode = 'options_found';
			}
			elsif (/^\s+([\w].*)/) {  # a continuation line
				$attrib[$#attrib] .= " $1";
				$mode = 'options_found';
			}

			# first blank line signals start of info
			# (strictly in 7.2, informally in 8.?
			elsif ( $mode eq 'options_found' and $_ =~ /^\s*$/) {
				$mode='info';
			}
			else { ; }     # blank lines before options, or else unrecognizable
		}
	}

	$info =~ s/^\s+//s;      # trim leading blanks
	$info =~ s/\s+$//s;      # trim trailing blanks

	($info) and push (@attrib, 'info'=>$info);

	# wants scalar?
	if ( $option ) {
		my %options = (@attrib);
		( $options{$option} ) and return $options{$option};

		$ILP::ERRMSG = "User option $option not defined";
		return ()
	}

	# wants array
	(@attrib)
}


1;

__END__

=head1 NAME

ILP - Perl 5 module to communicate with an interactive ListProc (ILP) server

=head1 DESCRIPTION

This module facilitates communication during a TCP/IP session with an
interactive ListProc server. The advantages of using this package vs.
directly communicating with the server are: Extraneous text, like the
'request>' prompt after each response, is removed; command output is
interpreted and converted into convenient scalars, lists, and hashes as
appropriate; and multiple ILP sessions can be managed easily, even when
open simultaneously.

=head1 USAGE

To use it in your programs, you can use either:

  use ILP;

or

  require ILP;

ILP exports nothing.

A new ILP object is created with the I<new> method.  Once
this has been done, all ILP commands are accessed through this object.

Here is a simple example of its usage. This script prints out the lists
a user is subscribed to:

  #!/usr/local/bin/perl
  
  use ILP;
  
  $ilp = new ILP('localhost', 'chad.redman@yale.edu', 'mypass')
           or die $ILP::ERRMSG;
  
  @mylists = $ilp->get_list_subs;
  
  if ( @mylists ) {
      print "Subscribed to:\n";
      print join("\n", @mylists)
  }
  else {
      print "Not subscribed to any lists!\n";
  }

  $ilp->close;
  
  __END__


=head2 ILP Methods

These methods are used to communicate with the server. For the most part, the
methods have two types of return values. For functions which perform an
action, the return value will be a 2-element array: the 3-digit code returned
by the server (100 generally means success; -100 is an internal code
indicating the command was rejected before being passed to the server), and
its corresponding text response. Thus, a scrupulous call to ILP::set_conceal
will capture the return value as:

  ($code, $response) = $my_ilp->set_conceal('ANNOUNCE-LIST', 'YES');
  if ($code != 100)  { warn "Command failed: $response" }

However, depending on what interests you, either the code or the response can
be captured individually:

  ($code) = $my_ilp->set_conceal('ANNOUNCE-LIST', 'YES');
  $resp   = $my_ilp->set_conceal('ANNOUNCE-LIST', 'YES');
     # (assigning list to a scalar--See Programming Perl, 1st ed., P. 73)


The other type of return value is for methods which give back specific
information. For example, ILP::get_user_subs() returns an array of all the
lists to which the connected user is subscribed. ILP::get_list_options($list,
$option) returns the scalar value for the key $option, or undef if not found.
In these cases, one cannot examine a 3-digit return code to determine
success. Depending on the expected result, an undefined scalar or a null
lists may indicate some kind of failure. Also, the class scalar $ILP::ERRMSG
will be set to a text error message for this type of function.


=over 10

=item I<new>

new( $host, $user, $pass ) returns ILP object

Create a new ILP object. This takes three arguments: a host, login name, and
password. The host can be of the form 'host:port', 'host' (defaults to
'host:372'), or '' (defaults to 'localhost:372'). If login name and password
are specified, attempt to log in. If not specified, log in as a casual user.

Upon successful socket creation, a reference to the created object is
returned. If there is a problem in setting up the socket, undef is returned,
and the error message is stored in the package variable $ILP::ERRMSG.

Examples:

  $ilp = new ILP('localhost:372', 'chad.redman@yale.edu', 'mypass')
              or die $ILP::ERRMSG;
or
  $ilp = new ILP('localhost') or die $ILP::ERRMSG;
or
  $ilp = new ILP or die $ILP::ERRMSG


=item I<request>

$ilp->request($cmd) returns ($code, $response)

Send the command in $cmd to the server as a single line. The 3-digit
code is returned in $code, and the server response is returned in
$response


=item I<do_command>

$ilp->do_command($cmd) returns ($code, $response)

A friendlier version of the request function. Issues the command in $cmd
to the server as a single line. Strip out unimportant text (the 'Your
command ... produced the following output' and the prompt 'request>')
and return the code with the remainder of the response.


=item I<close>

$ilp->close returns nothing

Issue a 'quit' to the server, and then drop the socket connection.
Further requests sent to the server (by the request() or do_command()
methods have no effect.


=item I<get_list_subs>

$ilp->get_list_subs() returns array of lists to which the user is subscribed

This method returns an array of the user's subscribed lists. These are local
names (without the '@domain' ending). Because it doesn't return the ($code,
$response) style here, the caller can't detect success based on returned
results; an empty list can still represent a successful command. Instead,
$ILP::ERRMSG will be set to code:response on error, and will be undefined on
success.


=item I<get_list_avail_names>

$ilp->get_list_avail
$ilp->get_list_avail('local')
    returns array of available lists

This method returns an array of lists available to the particular user. These
are domain-qualified addresses, in contrast to the values returned by
get_list_subs. Similar to that method, this doesn't return the ($code,
$response) style here. Instead, $ILP::ERRMSG will be set to code:response on
error, and will be undefined on success.


=item I<get_list_avail>

$ilp->get_list_avail
$ilp->get_list_avail('local')
    returns array of available lists

This method returns the same list as get_list_avail_names(), except
that the returned items will be of the form "list\tdescription" (i.e.,
the same response as is returned by the actual LISTS LOCAL command;
get_list_avail_names() edits this response).


=item I<set_password>

$ilp->set_password($list, $oldpass, $newpass) returns ($code, response)

Changes the user's password for the given list. A return code of 100
indicates success.


=item I<set_address>

$ilp->set_address($list, $pass, $newaddress) returns ($code, response)

This method changes the user's address for the given list. A code of 100
indicates success. I still need to investigate, but it seems that if you
change your address for one list, your login email address gets aliased to
the new one, and you can no longer change the address on other lists. Oops :(


=item I<set_mail>

$ilp->set_mail($list, $mail_setting) returns ($code, response)

Changes the user's mail preference for a list. Valid choices are:
ACK|NOACK|DIGEST|POSTPONE, but any value that is passed will be sent
to the server unchanged.


=item I<set_conceal>

$ilp->set_conceal($list, $conceal) returns ($code, response)

Changes the user's conceal flag for a list. Valid choices are: YES|NO


=item I<set_user_options>

$ilp->set_user_options($list, $option, @arguments, [$option, $arguments] ...)
    returns array of responses from each change

This is now deprecated in favor of the set_mail, set_conceal, etc. functions.
However, I'll leave it temporarily in in case someone finds it useful. This
is a way to specify multiple subscriber changes with a single call. E.g.,
@resps = $ilp->set_user_options('password'=>'oldpass'=>'newpass',
                          'mail'=>'ack', 'conceal'=>'yes')


=item I<get_user_options>

$ilp->get_user_options($list) returns hash of option=>value pairs for the
    specified list

Example:
  my %opts = $ilp->get_user_options('TEST-LIST')

The keys returned depend on the actual ListProc response, but can be expected
to be:
  address
  password
  mail
  conceal


=item I<get_list_options>

$ilp->get_list_options($list) returns hash of option=>value pairs
$ilp->get_list_options($list, $option) returns value for specified option

This method captures the output from an ILP 'REVIEW list SHORT' command, and
parses the results into a set of key=>value pairs. A sample array of
settings may have the following keys: ( * = special parsing performed to
obtain value )

 *address
 *description
 *created
 *info
  open
  send
  visible
  unpublished
  no-archive
  stats
  review
  archives
  unmoderated
  digest
  message-limit
  forward-rejects
 *reply-to-list-always (boolean)
  auto-delete-subscribers
  keep-resent-lines
  delivery-errors
  owners


=head1 COPYRIGHT

  Copyright 1999 Chad Redman, Yale University
               <chad.redman@yale.edu>

This library is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

=head1 AVAILABILITY

The latest version of this library is likely to be available from:

 http://pantheon.yale.edu/~cer28/projects/ilp/


=head1 VERSION

Version 0.5    (18 May, 1999)

=cut

